const C = Components.classes;
const I = Components.interfaces;

function eSwitchLocales_switch(event) {
  var prefs = C["@mozilla.org/preferences-service;1"].getService(I.nsIPrefBranch);
  var curLocale = "en-US";
  try {
    curLocale = prefs.getCharPref("general.useragent.locale");
  }
  catch (e) { }

  if (event.target.value != curLocale) {
    prefs.setCharPref("general.useragent.locale", event.target.value);

    var sbs = C["@mozilla.org/intl/stringbundle;1"].getService(I.nsIStringBundleService);
    var brand_sb = sbs.createBundle("chrome://global/locale/brand.properties");
    var ext_sb = sbs.createBundle("chrome://mozapps/locale/extensions/extensions.properties");
    var shortName = brand_sb.GetStringFromName("brandShortName");
    var promptStr = ext_sb.formatStringFromName("restartBeforeEnableMessage",
                                                [ event.target.label, shortName ], 2);

    var prompts = C["@mozilla.org/embedcomp/prompt-service;1"].getService(I.nsIPromptService);
    prompts.alert(window, document.getElementById("eSwitchLocales_menu").label,
                  promptStr);
  }
}

function eSwitchLocales_load(menu) {
  var rdfs = C["@mozilla.org/rdf/rdf-service;1"].getService(I.nsIRDFService);
  var rdfc = C["@mozilla.org/rdf/container-utils;1"].getService(I.nsIRDFContainerUtils);

  var prefs = C["@mozilla.org/preferences-service;1"].getService(I.nsIPrefBranch);
  var curLocale = "en-US";
  try {
    curLocale = prefs.getCharPref("general.useragent.locale");
  }
  catch (e) { }

  var chromeds = rdfs.GetDataSourceBlocking("rdf:chrome");

  var root = rdfs.GetResource("urn:mozilla:locale:root");

  var rPackages = rdfs.GetResource("http://www.mozilla.org/rdf/chrome#packages");
  var rName     = rdfs.GetResource("http://www.mozilla.org/rdf/chrome#name");
  var rPackage  = rdfs.GetResource("http://www.mozilla.org/rdf/chrome#package");
  var rDisplayName = rdfs.GetResource("http://www.mozilla.org/rdf/chrome#displayName");
  var rGlobal   = rdfs.GetResource("urn:mozilla:package:global");

  /* clear the existing children */
  var children = menu.childNodes;
  for (var i = children.length; i > 0; --i) {
    menu.removeChild(children[i - 1]);
  }

  var locales = rdfc.MakeSeq(chromeds, root).GetElements();
  var found = { };

  while (locales.hasMoreElements()) {
    var locale = locales.getNext().QueryInterface(I.nsIRDFResource);

    try {
      var name = chromeds.GetTarget(locale, rName, true).
        QueryInterface(I.nsIRDFLiteral).Value;

      if (found[name])
        continue;

      var displayName;
      try {
        displayName = " (" + chromeds.GetTarget(locale, rDisplayName, true).
          QueryInterface(I.nsIRDFLiteral).Value + ")";
      }
      catch (e) { dump(e); }

      var packagesr = chromeds.GetTarget(locale, rPackages, true);
      var packages = rdfc.MakeSeq(chromeds, packagesr).GetElements();

      while (packages.hasMoreElements()) {
        var provider = packages.getNext().QueryInterface(I.nsIRDFResource);
        if (chromeds.HasAssertion(provider, rPackage, rGlobal, true)) {
          var item = document.createElement("menuitem");
          item.setAttribute("value", name);
          item.setAttribute("label", name + displayName);
          item.setAttribute("name", "eLocaleSwitch");
          item.setAttribute("type", "radio");
          if (curLocale == name) {
            item.setAttribute("checked", "true");
          }

          menu.appendChild(item);
          item.addEventListener("command", eSwitchLocales_switch, false);

          found[name] = true;
          break;
        }
      }
    }
    catch (e) { dump(e); }
  }
}
